/**** Simple NIOS I/O Examples ****/

/* Example 5. Move a "blip" across 7 segment display
 * and green LEDs using simple subroutines with
 * "call" and "ret".
 *
 * The switches are used to control the speed.
 */

.include "ubc-de1media-macros.s"

.global _start

.text
_start:		movia r8, 0x00000001	/* initial blip pattern */
			movia	r23, IOBASE

loop:		mov	r4, r8	/* display the blip */
			call	display

			call	switches	/* switch values control the time delay */
			mov	r4, r2	/* parameter for delay = result of switches */
			call	delay

			roli	r8, r8, 1	/* rotate-left, how does this differ from slli? */
			
			br	loop


/* subroutine "display", to output the blip on the 7-seg and LEDG.
 * the input parameter is in "r4", the value to display.
 */
display:	stwio	r4, HEX7SEG(r23)
			stwio	r4, LEDG(r23)
			ret


/* subroutine "switches", to read the input switches.
 * the output result is the value read from the switches.
 */
switches:	ldwio	r2, SWITCH(r23)
			stwio	r2, LEDR(r23)	/* so user can see the switches on LEDR */
			ret


/* subroutine "delay", to waste some time
 * the input parameter is in "r4", the number of iterations
 * in the delay loop.
 */
delay:		movi	r2, 2000	/* multiply switch value (r4) by 2000 (r2) to create time delay */
			mul	r2, r2, r4	/* potential bug: when switches=0, delay gets very long because r2=0 */
			ori	r2, r2, 1	/* avoid potential bug: make sure r2 != 0 */
delay_loop:	addi	r2, r2, -1
			bne	r2, r0, delay_loop
			ret
		
.end
